@@ -27,20 +27,18 @@ THE SOFTWARE.
 import logging
 
 import os
-import stat as statinfo
-import time
+import shlex
 from SocketServer import StreamRequestHandler, ForkingTCPServer
 
 import paramiko
 from Crypto import Random
 
-from ftpcloudfs.fs import CloudFilesFS
+from ftpcloudfs.fs import ObjectStorageFS
 from ftpcloudfs.utils import smart_str
-from StringIO import StringIO
-from functools import wraps
+from sftpcloudfs.scp import SCPHandler
 
+from functools import wraps
 from posixpath import basename
-from sftpcloudfs.constants import version
 
 def return_sftp_errors(func):
     """
@@ -65,7 +63,7 @@ def return_sftp_errors(func):
 
 class SFTPServerInterface(paramiko.SFTPServerInterface):
     """
-    SFTPServerInterface implementation that exposes a CloudFilesFS object.
+    SFTPServerInterface implementation that exposes a ObjectStorageFS object.
     """
 
     def __init__(self, server, fs, *args, **kwargs):
@@ -128,7 +126,7 @@ class SFTPServerInterface(paramiko.SFTPServerInterface):
 
 class SFTPHandle(paramiko.SFTPHandle):
     """
-    Expose a CloudFilesFD object to SFTP.
+    Expose a ObjectStorageFD object to SFTP.
     """
 
     def __init__(self, owner, path, flags):
@@ -197,9 +195,9 @@ class SFTPHandle(paramiko.SFTPHandle):
         return paramiko.SFTP_OP_UNSUPPORTED
 
 
-class CloudFilesSFTPRequestHandler(StreamRequestHandler):
+class ObjectStorageSFTPRequestHandler(StreamRequestHandler):
     """
-    SocketServer RequestHandler subclass for CloudFilesSFTPServer.
+    SocketServer RequestHandler subclass for ObjectStorageSFTPServer.
 
     This RequestHandler subclass creates a paramiko Transport, sets up the
     sftp subsystem, and hands off to the transport's own request handling
@@ -232,22 +230,24 @@ class CloudFilesSFTPRequestHandler(StreamRequestHandler):
             self.log.warning("Channel is None, closing")
             t.close()
             return
+
         while t.isAlive():
             t.join(timeout=10)
 
-class CloudFilesSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
+class ObjectStorageSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
     """
-    Expose a CloudFilesFS object over SFTP.
+    Expose a ObjectStorageFS object over SFTP.
     """
     allow_reuse_address = True
 
-    def __init__(self, address, host_key=None, authurl=None, max_children=20, keystone=None):
+    def __init__(self, address, host_key=None, authurl=None, max_children=20, keystone=None, no_scp=False):
         self.log = paramiko.util.get_logger("paramiko")
         self.log.debug("%s: start server" % self.__class__.__name__)
-        self.fs = CloudFilesFS(None, None, authurl=authurl, keystone=keystone) # unauthorized
+        self.fs = ObjectStorageFS(None, None, authurl=authurl, keystone=keystone) # unauthorized
         self.host_key = host_key
         self.max_children = max_children
-        ForkingTCPServer.__init__(self, address, CloudFilesSFTPRequestHandler)
+        self.no_scp = no_scp
+        ForkingTCPServer.__init__(self, address, ObjectStorageSFTPRequestHandler)
 
     def check_channel_request(self, kind, chanid):
         if kind == 'session':
@@ -256,9 +256,35 @@ class CloudFilesSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
                          % (self.client_address, kind))
         # all the check_channel_*_request return False by default but
         # sftp subsystem because of the set_subsystem_handler call in
-        # the CloudFilesSFTPRequestHandler
+        # the ObjectStorageSFTPRequestHandler
         return paramiko.OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
 
+    def check_channel_exec_request(self, channel, command):
+        """Determine if a shell command will be executed for the client."""
+
+        # Parse the command
+        if ' -- ' in command:
+            # scp will use -- to delimit the begining of the unscaped filename
+            # so translate it to something that shelex can manage
+            command = command.replace(' -- ', ' "') + '"'
+        command = shlex.split(command)
+        self.log.debug('check_channel_exec_request %r' % command)
+
+        try:
+            if command[0] == 'scp':
+                if self.no_scp:
+                    self.log.info("scp exec request denied from=%s (scp is disabled)" % (self.client_address,))
+                    return False
+                self.log.info('invoking %r from=%s' % (command, self.client_address))
+                # handle the command execution
+                SCPHandler(command[1:], channel, self.fs, self.log).start()
+                return True
+        except:
+            self.log.exception("command %r failed from=%s" % (command, self.client_address))
+            return False
+
+        return False
+
     def check_auth_none(self, username):
         """Check whether the user can proceed without authentication."""
         return paramiko.AUTH_FAILED
@@ -275,12 +301,12 @@ class CloudFilesSFTPServer(ForkingTCPServer, paramiko.ServerInterface):
             if not password:
                 raise EnvironmentError("no password provided")
             self.fs.authenticate(username, password)
-            self.fs.connection.real_ip = self.client_address[0]
         except EnvironmentError, e:
             self.log.warning("%s: Failed to authenticate: %s" % (self.client_address, e))
             self.log.error("Authentication failure for %s from %s port %s" % (username,
                            self.client_address[0], self.client_address[1]))
             return paramiko.AUTH_FAILED
+        self.fs.conn.real_ip = self.client_address[0]
         self.log.info("%s authenticated from %s" % (username, self.client_address))
         return paramiko.AUTH_SUCCESSFUL
 
